﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections;

namespace LINQTests
{
    class Order
    {
        private int _OrderID;
        private int _CustomerID;
        private double _Cost;
        public int OrderID
        {
            get { return _OrderID; }
            set { _OrderID = value; }
        }
        public int CustomerID
        {
            get { return _CustomerID; }
            set { _CustomerID = value; }
        }
        public double Cost
        {
            get { return _Cost; }
            set { _Cost = value; }
        }
    }

    class Customer
    {
        private int _CustomerID;
        private string _Name;
        private string _Email;
        public int CustomerID
        {
            get { return _CustomerID; }
            set { _CustomerID = value; }
        }
        public string Name
        {
            get { return _Name; }
            set { _Name = value; }
        }
        public string Email
        {
            get { return _Email; }
            set { _Email = value; }
        }
    }

    class Program
    {
        static void Main(string[] args)
        {
            //set up some test customers
            var Customers = new List<Customer> {
                    new Customer {
                        CustomerID = 7,
                        Name = "Emma",
                        Email = "emz0r@worreva.com"
                    },
                    new Customer {
                        CustomerID = 84,
                        Name = "Pedro",
                        Email = "pedro@cerveza.es"
                    },
                    new Customer {
                        CustomerID = 102,
                        Name = "Vladimir",
                        Email = "vladimir@pivo.ru"
                    }
                };

            // Set up some test orders.
            var Orders = new List<Order> {
                         new Order {
                             OrderID = 1,
                             CustomerID = 84,
                             Cost = 159.12
                         },
                         new Order {
                             OrderID = 2,
                             CustomerID = 7,
                             Cost = 18.50
                         },
                         new Order {
                             OrderID = 3,
                             CustomerID = 84,
                             Cost = 2.89
                         },
                         new Order {
                             OrderID = 4,
                             CustomerID = 84,
                             Cost = 12.89
                         },
                         new Order {
                             OrderID = 5,
                             CustomerID = 84,
                             Cost = 9.99
                         },
                         new Order {
                             OrderID = 6,
                             CustomerID = 84,
                             Cost = 1.50
                         }
                     };

            TestFilter(Orders);
            TestAnonTypes(Orders);
            TestOrdering(Orders);
            TestJoin(Orders, Customers);
            TestSkipTake(Orders, Customers);
            TestElementOps(Orders, Customers);
            TestGroups(Orders, Customers);
            TestConversions(Orders, Customers);
            Console.ReadLine();
        }

        private static void TestFilter(List<Order> orders)
        {
            var col = from o in orders
                      where o.CustomerID == 84
                      select o;

            var col2 = orders.Where(o => o.CustomerID == 84);

            if (col.SequenceEqual(col2))
                Console.WriteLine("TestFilter Test Success");
            else
                Console.WriteLine("TestFilter Test Failure");
        }

        private static void TestAnonTypes(List<Order> orders)
        {
            var col = from o in orders
                      select new 
                      { 
                        o.OrderID, 
                        o.Cost 
                      };

            var col2 = orders.Select(o => new {OrderID = o.OrderID, Cost = o.Cost});

            if (col.SequenceEqual(col2))
                Console.WriteLine("TestAnonTypes Test Success");
            else
                Console.WriteLine("TestAnonTypes Test Failure");
        }

        private static void TestOrdering(List<Order> orders)
        {
            var col = from o in orders
                      orderby o.Cost ascending
                      select o;

            var col2 = orders.OrderBy(o => o.Cost);

            if (col.SequenceEqual(col2))
                Console.WriteLine("TestOrdering Test Success 1");
            else
                Console.WriteLine("TestOrdering Test Failure 1");



            var col3 = from o in orders
                      orderby o.Cost descending
                      select o;

            var col4 = orders.OrderByDescending(o => o.Cost);

            if (col3.SequenceEqual(col4))
                Console.WriteLine("TestOrdering Test Success 2");
            else
                Console.WriteLine("TestOrdering Test Failure 2");



            var col5 = from o in orders
                       orderby o.Cost descending
                       orderby o.CustomerID
                       select o;

            var col6 = orders.OrderBy(o => o.CustomerID).ThenByDescending(o => o.Cost);

            if (col5.SequenceEqual(col6))
                Console.WriteLine("TestOrdering Test Success 3");
            else
                Console.WriteLine("TestOrdering Test Failure 3");

            //Random Ordering
            Random R = new Random();
            var col7 = from o in orders
                       orderby R.Next()
                       select o;

            var col8 = orders.OrderBy(o => R.Next());
        }

        private static void TestJoin(List<Order> orders, List<Customer> customers)
        {
            var col = from c in customers
                      join o in orders on c.CustomerID equals o.CustomerID
                      select new {c.CustomerID, c.Name, o.OrderID, o.Cost};

            var col2 = customers.Join(orders, c => c.CustomerID, o => o.CustomerID, (c, o) => new { c.CustomerID, c.Name, o.OrderID, o.Cost });

            if (col.SequenceEqual(col2))
                Console.WriteLine("TestJoin Test Success");
            else
                Console.WriteLine("TestJoin Test Failure");
        }

        private static void TestSkipTake(List<Order> orders, List<Customer> customers)
        {
            //select top 3

            var col = (from o in orders
                       where o.CustomerID == 84
                       select o).Take(3);

            var col2 = orders.Where(o => o.CustomerID == 84).Take(3);

            if (col.SequenceEqual(col2))
                Console.WriteLine("TestSkipTake Test Success");
            else
                Console.WriteLine("TestSkipTake Test Failure");

            //skip first 2 records and return the 2 after

            var col3 = (from o in orders
                       where o.CustomerID == 84
                       orderby o.Cost
                       select o).Skip(2).Take(2);

            var col4 = orders.OrderBy(o => o.Cost).Where(o => o.CustomerID == 84).Skip(2).Take(2);

            if (col3.SequenceEqual(col4))
                Console.WriteLine("TestSkipTake2 Test Success");
            else
                Console.WriteLine("TestSkipTake2 Test Failure");
        }

        private static void TestElementOps(List<Order> orders, List<Customer> customers)
        {
            //returns a new customer instance if no elements
            var cust = (from c in customers
                       where c.CustomerID == 85
                       select c).DefaultIfEmpty(new Customer()).Single();

            var cust1 = customers.Where(c => c.CustomerID == 85).DefaultIfEmpty(new Customer()).Single();

            if (cust.CustomerID == cust1.CustomerID && cust1.CustomerID == 0)
                Console.WriteLine("TestElementOps1 Test Success");
            else
                Console.WriteLine("TestElementOps1 Test Failure");

            //returns a null if no elements
            var cust2 = (from c in customers
                        where c.CustomerID == 85
                        select c).SingleOrDefault();

            var cust3 = customers.Where(c => c.CustomerID == 85).SingleOrDefault();

            if (cust2 == cust3 && cust3 == null)
                Console.WriteLine("TestElementOps2 Test Success");
            else
                Console.WriteLine("TestElementOps2 Test Failure");


            //First, Last and ElementAt used in same way
            var cust4 = (from o in orders
                         where o.CustomerID == 84
                         orderby o.Cost
                         select o).Last();

            var cust5 = orders.Where(o => o.CustomerID == 84).OrderBy(o => o.Cost).Last();

            if (cust4 == cust5)
                Console.WriteLine("TestElementOps3 Test Success");
            else
                Console.WriteLine("TestElementOps3 Test Failure");
        }

        private static void TestGroups(List<Order> orders, List<Customer> customers)
        {
            var OrderCounts = from o in orders
                              group o by o.CustomerID into g
                              select new
                              {
                                  CustomerID = g.Key,
                                  TotalOrders = g.Count()
                              };

            var OrderCounts1 = orders.GroupBy(o => o.CustomerID).Select(g => new { CustomerID = g.Key, TotalOrders = g.Count() });

            if (OrderCounts.Count() == OrderCounts1.Count())
                Console.WriteLine("TestGroups Test Success");
            else
                Console.WriteLine("TestGroups Test Failure");

        }


        private static void TestConversions(List<Order> orders, List<Customer> customers)
        {
            string[] names = (from c in customers
                              select c.Name).ToArray();

            Dictionary<int, Customer> col = customers.ToDictionary(c => c.CustomerID);

            Dictionary<string, double> customerOrdersWithMaxCost = (from oc in
                                                                     (from o in orders
                                                                      join c in customers on o.CustomerID equals c.CustomerID
                                                                      select new { c.Name, o.Cost })
                                                                 group oc by oc.Name into g
                                                                 select g).ToDictionary(g => g.Key, g => g.Max(oc => oc.Cost));
                                            //group o by  into g
                                            //select g).ToDictionary(g => g.Key, g => g.Max(o => o.Cost));

            List<Order> ordersOver10 = (from o in orders
                                       where o.Cost > 10
                                       orderby o.Cost
                                       select o).ToList();

            ILookup<int, string> customerLookup = customers.ToLookup(c => c.CustomerID, c => c.Name);
                                        

            Console.WriteLine("TestConversions Test Failure");
        }
    }
}
